from __future__ import with_statement
from os.path import *
import sys, doctest, unittest, imp
from contextlib import contextmanager
from nose.loader import TestLoader as NoseTestLoader
from nose.config import Config as NoseConfig
from nose.plugins.manager import PluginManager
from nose.plugins.capture import Capture
#from nose.plugins.doctests import DocFileCase

__all__ = ['TestSuite']

mod_suite = imp.new_module('_test_suite')
sys.modules['_test_suite'] = mod_suite

class TestSuite(unittest.TestSuite):
    doctest_flags = doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE | doctest.REPORT_ONLY_FIRST_FAILURE
    doctest_globs = {'with_statement' : with_statement}

    def __init__(self, tests_dir, coverage=False, pkg=None, exclude=()):
        super(TestSuite, self).__init__()
        plugin_manager = PluginManager([Capture()])
        nose_config = NoseConfig(plugins=plugin_manager)
        self.nose_loader = NoseTestLoader(config=nose_config)
        import __main__ as mod_main
        self.script_dir = dirname(abspath(mod_main.__file__))
        sys.path.insert(0, self.script_dir)

        self.tests_dir = join(self.script_dir, tests_dir)
        self.pkg = pkg
        self.coverage = False
        if coverage:
            try:
                from coverage import coverage as coverage_cls
            except ImportError:
                pass
            else:
                self.cov = coverage_cls(branch=True)
                self.cov.exclude('def __repr__')
                #cov.exclude('#pragma[: ]+[nN][oO] [cC][oO][vV][eE][rR]')
                self.cov.start()
                #import mext.reaction
                self.coverage = True
                self.coverage_dir = join(self.script_dir, coverage)
                self.coverage_exclude = set(exclude)


    def add_doctest(self, name):
        path = join(self.tests_dir, name + '.txt')
        case = doctest.DocFileTest(
            path,
            module_relative=False,
            optionflags=self.doctest_flags,
            globs=self.doctest_globs
        )
        #test = doctest.DocFileTest(path, optionflags=flags, globs=globs)
        #case = DocFileCase(test)
        self.addTest(case)

    def add_nosetest(self, *args, **kw):
        assert 'nose' not in kw
        kw['nose'] = True
        self.add_unittest(*args, **kw)

    def add_unittest(self, name, optional=False, nose=False):
        name = name.replace('.', '/')
        with override_sys_path():
            fname = join(self.tests_dir, name+'.py')
            code = open(fname, 'rb').read()
            code = compile(code, fname, 'exec')
            mname = '_test_suite'
            parent = mod_suite
            for seg in name.split('/'):
                mname += '.'+seg
                mod = sys.modules.setdefault(mname, imp.new_module(mname))
                setattr(parent, seg, mod)
                parent = mod

            mod.__file__ = fname
            #mod.__path__ = [self.tests_dir]
            try:
                exec code in mod.__dict__
            except ImportError:
                if optional:
                    return
                else:
                    raise
            if nose:
                case = self.nose_loader.loadTestsFromModule(mod)
            else:
                case = unittest.findTestCases(mod)
            self.addTest(case)


    def run_text(self, verbose=False):
        runner = unittest.TextTestRunner(verbosity=(2 if verbose else 0))
        runner.run(self)
        if self.coverage:
            self.cov.stop()
            morfs = []
            for name, mod in sys.modules.items():
                if not mod or name in self.coverage_exclude:
                    continue
                elif self.pkg and not name.startswith(self.pkg):
                    continue
                else:
                    morfs.append(mod)
            self.cov.html_report(morfs, directory=self.coverage_dir)#, ignore_errors=True)
        return runner




@contextmanager
def override_sys_path():
    sys_path = sys.path[:]
    try:
        yield
    finally:
        sys.path = sys_path

